@@ -8,7 +8,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: data.h,v 1.76 2001/12/14 10:09:51 strauss Exp $
+ * @(#) $Id: data.h 1553 2003-04-30 14:29:31Z strauss $
  */
 
 #ifndef _DATA_H
@@ -43,6 +43,7 @@ typedef enum Kind {
 
 typedef unsigned short ParserFlags;
 typedef unsigned short ModuleFlags;
+typedef unsigned short ImportFlags;
 typedef unsigned short ObjectFlags;
 typedef unsigned short NodeFlags;
 typedef unsigned short TypeFlags;
@@ -57,6 +58,11 @@ typedef unsigned short MacroFlags;
 #define	FLAG_CREATABLE	        0x0040 /* On a Row: New rows can be created. */
 #define FLAG_INGROUP		0x0080 /* Node is contained in a group.      */
 #define	FLAG_INCOMPLIANCE	0x0100 /* Group is mentioned in a compliance
+                                           statement.
+					   In case of ImportFlags: the import
+				           is done through a compliance
+				           MODULE phrase.                    */
+#define	FLAG_INSYNTAX		0x0200 /* Type is mentioned in a syntax
                                            statement.                        */
 
 
@@ -106,6 +112,7 @@ typedef struct Revision {
 typedef struct Import {
     SmiImport      export;
     Module         *modulePtr;
+    ImportFlags	   flags;
     struct Import  *nextPtr;
     struct Import  *prevPtr;
     Kind	   kind;
@@ -146,6 +153,7 @@ typedef struct Option {
     SmiOption      export;
     struct Object  *compliancePtr;
     struct Object  *objectPtr;
+    int		   line;
 } Option;
 
 
@@ -156,6 +164,7 @@ typedef struct Refinement {
     struct Object  *objectPtr;     /* the Object refined by this Refinement  */
     Type	   *typePtr;       /* the refined Type (or NULL)             */
     Type	   *writetypePtr;  /* the refined WriteType (or NULL)        */
+    int		   line;
 } Refinement;
 
 
@@ -176,6 +185,11 @@ typedef struct Index {
 } Index;
 
 
+typedef struct SubjectCategories {
+    struct List *categories;
+    int         allCategories;
+} SubjectCategories;
+
 
 typedef struct Object {
     SmiNode        export;
@@ -191,6 +205,7 @@ typedef struct Object {
     struct Object  *nextPtr;
     struct Object  *prevSameNodePtr;    /* chain of Objects for this Node  */
     struct Object  *nextSameNodePtr;
+    struct List    *uniquenessPtr;      /* UNIQUENESS */
     int		   line;
 } Object;
 
@@ -329,6 +344,8 @@ extern void setRevisionLine(Revision *revisionPtr,
 extern Import *addImport(char *name,
 			 Parser *parserPtr);
 
+extern void addImportFlags(Import *importPtr, ImportFlags flags);
+
 extern void setImportModulename(Import *importPtr,
 				char *modulename);
 
@@ -391,6 +408,10 @@ extern void setObjectReference(Object *objectPtr,
 extern void setObjectDecl(Object *objectPtr,
 			   SmiDecl decl);
 
+extern void setObjectUniqueness(Object *objectPtr, List *listPtr);
+
+extern void setObjectInstallErrors(Object *objectPtr, List *listPtr);
+
 extern void setObjectLine(Object *objectPtr,
 			  int line,
 			  Parser *parserPtr);
@@ -567,7 +588,7 @@ extern int smiInitData(void);
 
 extern void smiFreeData(void);
 
-extern Module *loadModule(const char *modulename);
+extern Module *loadModule(const char *modulename, Parser *parserPtr);
 
 
 #endif /* _DATA_H */
