@@ -8,7 +8,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: smi.c,v 1.106 2001/10/08 14:16:57 schoenw Exp $
+ * @(#) $Id: smi.c 1557 2003-05-12 17:31:19Z tklie $
  */
 
 #include <config.h>
@@ -29,6 +29,7 @@
 #include "data.h"
 #include "error.h"
 #include "util.h"
+#include "snprintf.h"
 
 #ifdef BACKEND_SMI
 #include "scanner-smi.h"
@@ -228,9 +229,8 @@ int smiInit(const char *tag)
 	pw = getpwuid(getuid());
 	if (pw && pw->pw_dir) {
 	    /* 3. read user config file if present (append/prepend/replace) */
-	    p = smiMalloc(strlen(DEFAULT_USERCONFIG) +
-			    strlen(pw->pw_dir) + 2);
-	    sprintf(p, "%s%c%s", pw->pw_dir, DIR_SEPARATOR, DEFAULT_USERCONFIG);
+	    smiAsprintf(&p, "%s%c%s",
+			pw->pw_dir, DIR_SEPARATOR, DEFAULT_USERCONFIG);
 	    smiReadConfig(p, tag2);
 	    smiFree(p);
 	}
@@ -242,13 +242,11 @@ int smiInit(const char *tag)
     p = getenv("SMIPATH");
     if (p) {
 	if (p[0] == PATH_SEPARATOR) {
-	    pp = smiMalloc(strlen(p) + strlen(smiHandle->path) + 1);
-	    sprintf(pp, "%s%s", smiHandle->path, p);
+	    smiAsprintf(&pp, "%s%s", smiHandle->path, p);
 	    smiFree(smiHandle->path);
 	    smiHandle->path = pp;
 	} else if (p[strlen(p)-1] == PATH_SEPARATOR) {
-	    pp = smiMalloc(strlen(p) + strlen(smiHandle->path) + 1);
-	    sprintf(pp, "%s%s", p, smiHandle->path);
+	    smiAsprintf(&pp, "%s%s", p, smiHandle->path);
 	    smiFree(smiHandle->path);
 	    smiHandle->path = pp;
 	} else {
@@ -301,6 +299,8 @@ int smiSetPath(const char *s)
 {
     char *s2;
 
+    if (!smiHandle) smiInit(NULL);
+
     if (!s) {
 	smiFree(smiHandle->path);
 	smiHandle->path = NULL;
@@ -353,13 +353,11 @@ int smiReadConfig(const char *filename, const char *tag)
 	    } else if (!strcmp(cmd, "path")) {
 		if (arg) {
 		    if (arg[0] == PATH_SEPARATOR) {
-			s = smiMalloc(strlen(arg) + strlen(smiHandle->path) + 1);
-			sprintf(s, "%s%s", smiHandle->path, arg);
+			smiAsprintf(&s, "%s%s", smiHandle->path, arg);
 			smiFree(smiHandle->path);
 			smiHandle->path = s;
 		    } else if (arg[strlen(arg)-1] == PATH_SEPARATOR) {
-			s = smiMalloc(strlen(arg) + strlen(smiHandle->path) + 1);
-			sprintf(s, "%s%s", arg, smiHandle->path);
+			smiAsprintf(&s, "%s%s", arg, smiHandle->path);
 			smiFree(smiHandle->path);
 			smiHandle->path = s;
 		    } else {
@@ -415,7 +413,7 @@ char *smiLoadModule(const char *module)
 
     if (smiIsPath(module)) {
 
-	modulePtr = loadModule(module);
+	modulePtr = loadModule(module, NULL);
 
 	if (modulePtr) {
 	    if (!isInView(modulePtr->export.name)) {
@@ -428,15 +426,17 @@ char *smiLoadModule(const char *module)
 
     } else {
 	
-	if (!isInView(module)) {
-	    addView(module);
-	}
-	
 	if ((modulePtr = findModuleByName(module))) {
 	    /* already loaded. */
+	    if (!isInView(module)) {
+		addView(module);
+	    }
 	    return modulePtr->export.name;
 	} else {
-	    if ((modulePtr = loadModule(module))) {
+	    if ((modulePtr = loadModule(module, NULL))) {
+		if (!isInView(module)) {
+		    addView(module);
+		}
 		return modulePtr->export.name;
 	    } else {
 		return NULL;
@@ -474,64 +474,6 @@ int smiGetFlags()
 
 
 
-#if 0
-char *smiModule(char *fullname)
-{
-    int	 len;
-    char *module;
-
-    if (!fullname) {
-	return NULL;
-    }
-
-    if (islower((int)fullname[0])) {
-	return NULL;
-    }
-    
-    len = strcspn(fullname, "!.");
-    len = MIN(len, strcspn(fullname, "::"));
-    module = smiStrndup(fullname, len);
-
-    return module;
-}
-
-
-
-char *smiDescriptor(char *fullname)
-{
-    char *p, *name;
-
-    if (!fullname) {
-	return NULL;
-    }
-    
-    p = strstr(fullname, "::");
-    if (p) {
-	name = strdup(&p[2]);
-    } else {
-	p = strchr(fullname, '!');
-	if (p) {
-	    name = strdup(&p[1]);
-	} else {
-	    if (!isupper((int)fullname[0])) {
-		name = strdup(fullname);
-	    } else {
-		p = strchr(fullname, '.');
-		if (p) {
-		    name = strdup(&p[1]);
-		} else {
-		    name = strdup(fullname);
-		}
-	    }
-	}
-    }
-	 
-    return name;
-}
-#endif
-
-
-
 SmiModule *smiGetModule(const char *module)
 {
     Module	      *modulePtr;
@@ -543,7 +485,7 @@ SmiModule *smiGetModule(const char *module)
     modulePtr = findModuleByName(module);
     
     if (!modulePtr) {
-	modulePtr = loadModule(module);
+	modulePtr = loadModule(module, NULL);
     }
     
     return &modulePtr->export;
@@ -686,7 +628,7 @@ SmiType *smiGetType(SmiModule *smiModulePtr, char *type)
     
     if (!modulePtr && module2 && strlen(module2)) {
 	if (!(modulePtr = findModuleByName(module2))) {
-	    modulePtr = loadModule(module2);
+	    modulePtr = loadModule(module2, NULL);
 	}
     }
 
@@ -900,7 +842,7 @@ SmiMacro *smiGetMacro(SmiModule *smiModulePtr, char *macro)
     
     if (!modulePtr && module2 && strlen(module2)) {
 	if (!(modulePtr = findModuleByName(module2))) {
-	    modulePtr = loadModule(module2);
+	    modulePtr = loadModule(module2, NULL);
 	}
     }
 
@@ -966,7 +908,7 @@ SmiNode *smiGetNode(SmiModule *smiModulePtr, const char *node)
     
     if (!modulePtr && module2 && strlen(module2)) {
 	if (!(modulePtr = findModuleByName(module2))) {
-	    modulePtr = loadModule(module2);
+	    modulePtr = loadModule(module2, NULL);
 	}
     }
 
@@ -1168,11 +1110,11 @@ SmiNode *smiGetParentNode(SmiNode *smiNodePtr)
     if (!objectPtr) {
 	objectPtr = findObjectByNode(nodePtr);
 
-	if (!objectPtr) {
-	    /* in implicitly created node, e.g. gaga.0 in an object
+	if ((!objectPtr) && (nodePtr->parentPtr)) {
+	    /* an implicitly created node, e.g. gaga.0 in an object
 	     * definition with oid == gaga.0.1.
 	     */
-	    objectPtr = addObject("<implicit>",
+	    objectPtr = addObject(SMI_UNKNOWN_LABEL,
 				  nodePtr->parentPtr, nodePtr->subid,
 				  0, NULL);
 	    objectPtr->nodePtr = nodePtr;
@@ -1292,6 +1234,8 @@ SmiType *smiGetNodeType(SmiNode *smiNodePtr)
     return &typePtr->export;
 }
 
+
+
 int smiGetNodeLine(SmiNode *smiNodePtr)
 {
     return ((Object *)smiNodePtr)->line;
@@ -1331,7 +1275,10 @@ SmiElement *smiGetNextElement(SmiElement *smiElementPtr)
 
 SmiNode *smiGetElementNode(SmiElement *smiElementPtr)
 {
-    return &((Object *)((List *)smiElementPtr)->ptr)->export;
+    if ((Object *)((List *)smiElementPtr)->ptr)
+        return &((Object *)((List *)smiElementPtr)->ptr)->export;
+    else
+        return NULL;
 }
 
 
@@ -1392,6 +1339,13 @@ SmiNode *smiGetOptionNode(SmiOption *smiOptionPtr)
 
 
 
+int smiGetOptionLine(SmiOption *smiOptionPtr)
+{
+    return ((Option *)smiOptionPtr)->line;
+}
+
+
+
 SmiRefinement *smiGetFirstRefinement(SmiNode *smiComplianceNodePtr)
 {
     Object	      *objectPtr;
@@ -1477,3 +1431,574 @@ SmiType *smiGetRefinementWriteType(SmiRefinement *smiRefinementPtr)
     
     return &typePtr->export;
 }
+
+
+
+int smiGetRefinementLine(SmiRefinement *smiRefinementPtr)
+{
+    return ((Refinement *)smiRefinementPtr)->line;
+}
+
+
+
+SmiElement *smiGetFirstUniquenessElement(SmiNode *smiNodePtr)
+{
+    List	      *listPtr;
+    
+    if (!smiNodePtr) {
+	return NULL;
+    }
+
+    listPtr = ((Object *)smiNodePtr)->uniquenessPtr;
+
+    return (SmiElement *)listPtr;
+}
+
+
+
+char *smiRenderOID(unsigned int oidlen, SmiSubid *oid, int flags)
+{
+    SmiNode *nodePtr;
+    SmiModule *modulePtr = NULL;
+    unsigned int i = 0;
+    char *ss, *s = NULL;
+
+    if (!oid) {
+	if (flags & SMI_RENDER_UNKNOWN) {
+	    smiAsprintf(&s, SMI_UNKNOWN_LABEL);
+	} else {
+	    s = NULL;
+	}
+	return s;
+    }
+    
+    if (flags & (SMI_RENDER_NAME | SMI_RENDER_QUALIFIED)) {
+	nodePtr = smiGetNodeByOID(oidlen, oid);
+	if (nodePtr) {
+	    i = nodePtr->oidlen;
+	    if (flags & SMI_RENDER_QUALIFIED) {
+		modulePtr = smiGetNodeModule(nodePtr);
+	    }
+	    if (modulePtr) {
+		smiAsprintf(&s, "%s::%s",
+			    modulePtr->name, nodePtr->name);
+	    } else {
+		smiAsprintf(&s, "%s", nodePtr->name);
+	    }
+	}
+    }
+
+    for (; i < oidlen; i++) {
+	ss = s;
+	smiAsprintf(&s, "%s%s%u", ss ? ss : "", i ? "." : "", oid[i]);
+	smiFree(ss);
+    }
+
+    if ((!s) && (flags & SMI_RENDER_UNKNOWN)) {
+	smiAsprintf(&s, SMI_UNKNOWN_LABEL);
+    }
+    
+    return s;
+}
+
+
+
+char *smiRenderValue(SmiValue *smiValuePtr, SmiType *smiTypePtr, int flags)
+{
+    int i, j, k, n, pfx, have_pfx;
+    char *last_fmt, *fmt;
+    SmiUnsigned64 vv;
+    int xlen;
+    SmiNamedNumber *nn;
+    char *s, *ss;
+    char f[8];
+    SmiUnsigned32 v32;
+    SmiUnsigned64 v64;
+    
+    if (!smiValuePtr) {
+	if (flags & SMI_RENDER_UNKNOWN) {
+	    smiAsprintf(&s, SMI_UNKNOWN_LABEL);
+	} else {
+	    s = NULL;
+	}
+	return s;
+    }
+    
+    switch (smiValuePtr->basetype) {
+    case SMI_BASETYPE_UNSIGNED32:
+	if (!(flags & SMI_RENDER_FORMAT) ||
+	    !smiTypePtr || !smiTypePtr->format ||
+	    !strlen(smiTypePtr->format) || smiTypePtr->format[0] == 'd') {
+	    if (smiTypePtr->format && (strlen(smiTypePtr->format) >= 3) &&
+		(smiTypePtr->format[1] == '-')) {
+		i = atoi(&smiTypePtr->format[2]);
+		if (i < 0) i = 0;
+		if (i > 20) i = 20;
+		smiAsprintf(&s, "%0*lu.",
+			    1 + i,
+			    smiValuePtr->value.unsigned32);
+		if (s) {
+		    for (j = strlen(s) - 1; i > 0; i--, j--) {
+			s[j] = s[j-1];
+		    }
+		    s[j] = '.';
+		}
+	    } else {
+		smiAsprintf(&s, "%lu", smiValuePtr->value.unsigned32);
+	    }
+	} else if (smiTypePtr->format[0] == 'x') {
+	    smiAsprintf(&s, "%lx", smiValuePtr->value.unsigned32);
+	} else if (smiTypePtr->format[0] == 'o') {
+	    smiAsprintf(&s, "%lo", smiValuePtr->value.unsigned32);
+	} else if (smiTypePtr->format[0] == 'b') {
+	    for (i = 32 - 1;
+		 i > 0 && !(smiValuePtr->value.unsigned32 & (1 << i)); i--);
+	    s = smiMalloc(i + 1 + 1);
+	    if (s) {
+		for (j = 0; i >= 0; i--, j++) {
+		    s[j] = smiValuePtr->value.unsigned32 & (1<<i) ? '1' : '0';
+		}
+		s[j] = 0;
+	    }
+	}
+	break;
+    case SMI_BASETYPE_UNSIGNED64:
+	if (!(flags & SMI_RENDER_FORMAT) ||
+	    !smiTypePtr || !smiTypePtr->format ||
+	    !strlen(smiTypePtr->format) || smiTypePtr->format[0] == 'd') {
+	    if (smiTypePtr->format && (strlen(smiTypePtr->format) >= 3) &&
+		(smiTypePtr->format[1] == '-')) {
+		i = atoi(&smiTypePtr->format[2]);
+		if (i < 0) i = 0;
+		if (i > 20) i = 20;
+		sprintf(f, "%%0%s.", UINT64_FORMAT);
+		f[2] = '*';
+		smiAsprintf(&s, f,
+			    1 + i,
+			    smiValuePtr->value.unsigned64);
+		if (s) {
+		    for (j = strlen(s) - 1; i > 0; i--, j--) {
+			s[j] = s[j-1];
+		    }
+		    s[j] = '.';
+		}
+	    } else {
+		smiAsprintf(&s, UINT64_FORMAT, smiValuePtr->value.unsigned64);
+	    }
+	} else if (smiTypePtr->format[0] == 'x') {
+	    strcpy(f, UINT64_FORMAT);
+	    f[strlen(f)-1] = 'x';
+	    smiAsprintf(&s, f, smiValuePtr->value.unsigned64);
+	} else if (smiTypePtr->format[0] == 'o') {
+	    strcpy(f, UINT64_FORMAT);
+	    f[strlen(f)-1] = 'o';
+	    smiAsprintf(&s, f, smiValuePtr->value.unsigned64);
+	} else if (smiTypePtr->format[0] == 'b') {
+	    for (i = 64 - 1;
+		 i > 0 && !(smiValuePtr->value.unsigned64 & (1 << i)); i--);
+	    s = smiMalloc(i + 1 + 1);
+	    if (s) {
+		for (j = 0; i >= 0; i--, j++) {
+		    s[j] = smiValuePtr->value.unsigned64 & (1<<i) ? '1' : '0';
+		}
+		s[j] = 0;
+	    }
+	}
+	break;
+    case SMI_BASETYPE_INTEGER32:
+	if (!(flags & SMI_RENDER_FORMAT) ||
+	    !smiTypePtr || !smiTypePtr->format ||
+	    !strlen(smiTypePtr->format) || smiTypePtr->format[0] == 'd') {
+	    if (smiTypePtr->format && (strlen(smiTypePtr->format) >= 3) &&
+		(smiTypePtr->format[1] == '-')) {
+		i = atoi(&smiTypePtr->format[2]);
+		if (i < 0) i = 0;
+		if (i > 20) i = 20;
+		smiAsprintf(&s, "%0*ld.",
+			    1 + i + (smiValuePtr->value.integer32 < 0 ? 1 : 0),
+			    smiValuePtr->value.integer32);
+		if (s) {
+		    for (j = strlen(s) - 1; i > 0; i--, j--) {
+			s[j] = s[j-1];
+		    }
+		    s[j] = '.';
+		}
+	    } else {
+		smiAsprintf(&s, "%ld", smiValuePtr->value.integer32);
+	    }
+	} else if (smiTypePtr->format[0] == 'x') {
+	    if (smiValuePtr->value.integer32 >= 0) {
+		smiAsprintf(&s, "%lx", smiValuePtr->value.integer32);
+	    } else {
+		smiAsprintf(&s, "-%lx", - smiValuePtr->value.integer32);
+	    }
+	} else if (smiTypePtr->format[0] == 'o') {
+	    if (smiValuePtr->value.integer32 >= 0) {
+		smiAsprintf(&s, "%lo", smiValuePtr->value.integer32);
+	    } else {
+		smiAsprintf(&s, "-%lo", - smiValuePtr->value.integer32);
+	    }
+	} else if (smiTypePtr->format[0] == 'b') {
+	    if (smiValuePtr->value.integer32 >= 0) {
+		v32 = smiValuePtr->value.integer32;
+		j = 0;
+	    } else {
+		v32 = - smiValuePtr->value.integer32;
+		j = 1;
+	    }
+	    for (i = 32 - 1;
+		 i > 0 && !(v32 & (1 << i)); i--);
+	    s = smiMalloc(i + j + 1 + 1);
+	    if (s) {
+		s[0] = '-';
+		for (; i >= 0; i--, j++) {
+		    s[j] = v32 & (1<<i) ? '1' : '0';
+		}
+		s[j] = 0;
+	    }
+	}
+	break;
+    case SMI_BASETYPE_INTEGER64:
+	if (!(flags & SMI_RENDER_FORMAT) ||
+	    !smiTypePtr || !smiTypePtr->format ||
+	    !strlen(smiTypePtr->format) || smiTypePtr->format[0] == 'd') {
+	    if (smiTypePtr->format && (strlen(smiTypePtr->format) >= 3) &&
+		(smiTypePtr->format[1] == '-')) {
+		i = atoi(&smiTypePtr->format[2]);
+		if (i < 0) i = 0;
+		if (i > 20) i = 20;
+		sprintf(f, "%%0%s.", INT64_FORMAT);
+		f[2] = '*';
+		smiAsprintf(&s, f,
+			    1 + i + (smiValuePtr->value.integer64 < 0 ? 1 : 0),
+			    smiValuePtr->value.integer64);
+		if (s) {
+		    for (j = strlen(s) - 1; i > 0; i--, j--) {
+			s[j] = s[j-1];
+		    }
+		    s[j] = '.';
+		}
+	    } else {
+		smiAsprintf(&s, INT64_FORMAT, smiValuePtr->value.integer64);
+	    }
+	} else if (smiTypePtr->format[0] == 'x') {
+	    if (smiValuePtr->value.integer64 >= 0) {
+		strcpy(f, UINT64_FORMAT);
+		f[strlen(f)-1] = 'x';
+		smiAsprintf(&s, f, smiValuePtr->value.integer64);
+	    } else {
+		sprintf(f, "-%s", UINT64_FORMAT);
+		f[strlen(f)-1] = 'x';
+		smiAsprintf(&s, f, - smiValuePtr->value.integer64);
+	    }
+	} else if (smiTypePtr->format[0] == 'o') {
+	    if (smiValuePtr->value.integer64 >= 0) {
+		strcpy(f, UINT64_FORMAT);
+		sprintf(f, "-%s", UINT64_FORMAT);
+		f[strlen(f)-1] = 'o';
+		smiAsprintf(&s, f, smiValuePtr->value.integer64);
+	    } else {
+		smiAsprintf(&s, f, - smiValuePtr->value.integer64);
+	    }
+	} else if (smiTypePtr->format[0] == 'b') {
+	    if (smiValuePtr->value.integer64 >= 0) {
+		v64 = smiValuePtr->value.integer64;
+		j = 0;
+	    } else {
+		v64 = - smiValuePtr->value.integer64;
+		j = 1;
+	    }
+	    for (i = 64 - 1;
+		 i > 0 && !(v64 & (1 << i)); i--);
+	    s = smiMalloc(i + j + 1 + 1);
+	    if (s) {
+		s[0] = '-';
+		for (; i >= 0; i--, j++) {
+		    s[j] = v64 & (1<<i) ? '1' : '0';
+		}
+		s[j] = 0;
+	    }
+	}
+	break;
+    case SMI_BASETYPE_OBJECTIDENTIFIER:
+	s = smiRenderOID(smiValuePtr->len, smiValuePtr->value.oid, flags);
+	break;
+    case SMI_BASETYPE_OCTETSTRING:
+	if (!(flags & SMI_RENDER_FORMAT) ||
+	    (!smiTypePtr->format &&
+	     (smiTypePtr->name && strcmp( smiTypePtr->name, "IpAddress")) ) ) {
+	    for (i = 0; i < smiValuePtr->len; i++) {
+		if (!isprint((int)smiValuePtr->value.ptr[i])) break;
+	    }
+	    if ((i < smiValuePtr->len) ||
+		!(flags & SMI_RENDER_PRINTABLE)) {
+		smiAsprintf(&s, "");
+		for (i=0; i < smiValuePtr->len; i++) {
+		    ss = s;
+		    smiAsprintf(&s, "%s%02x", ss, smiValuePtr->value.ptr[i]);
+		    smiFree(ss);
+		}
+	    } else {
+		smiAsprintf(&s, "%s", smiValuePtr->value.ptr);
+	    }
+	} else {
+	    i = 0;
+	    smiAsprintf(&s, "");
+	    /* SNMPv2-SMI:IpAddress does not have a display hint.
+	       ==> let's use this one: "1d." if we have an IpAddress here */
+	    fmt = (smiTypePtr->name &&
+		   strcmp( smiTypePtr->name, "IpAddress" ) ) ?
+		smiTypePtr->format : "1d.";
+	    while (*fmt && i < smiValuePtr->len) {
+		last_fmt = fmt;
+		have_pfx = pfx = 0; /* scan prefix: */
+		while (*fmt && isdigit((int)*fmt)) {
+		    pfx = pfx * 10 + *fmt - '0', have_pfx = 1, fmt++;
+		}
+		if (! have_pfx) {
+		    pfx = 1;
+		}
+		switch (*fmt) {
+		case 't':
+		    /* XXX UTF-8 not implemented, fall through to ASCII (a) */
+		case 'a':
+		    n = (pfx < (smiValuePtr->len - i)) ?
+			       pfx : smiValuePtr->len - i;
+		    for (k = 0; k < n; k++) {
+			if (! isascii((int) smiValuePtr->value.ptr[i+k])) {
+			    smiFree(s);
+			    if (flags & SMI_RENDER_UNKNOWN) {
+				smiAsprintf(&s, SMI_LANGUAGE_UNKNOWN);
+			    } else {
+				s = NULL;
+			    }
+			    return s;
+			}
+			ss = s;
+			smiAsprintf(&s, "%s%c", ss, smiValuePtr->value.ptr[i+k]);
+			smiFree(ss);
+		    }
+		    i += n;
+		    break;
+		case 'b':
+		case 'd':
+		case 'o':
+		case 'x':
+		    /* XXX: limited to no more than
+		       sizeof(SmiUnsigned64) octets */
+		    vv = 0;
+		    xlen = pfx * 2;
+		    while (pfx > 0 && i < smiValuePtr->len) {
+			vv = vv * 256 +
+			     ((unsigned char)smiValuePtr->value.ptr[i]);
+			i++;
+			pfx--;
+		    }
+		    switch (*fmt) {
+		    case 'd':
+			ss = s;
+			sprintf(f, "%%s%s", UINT64_FORMAT);
+			smiAsprintf(&s, f, ss, vv);
+			smiFree(ss);
+			break;
+		    case 'o':
+			ss = s;
+			sprintf(f, "%%s%s", UINT64_FORMAT);
+			f[strlen(f)-1] = 'o';
+			smiAsprintf(&s, f, ss, vv);
+			smiFree(ss);
+			break;
+		    case 'x':
+			ss = s;
+			sprintf(f, "%%s%%0%s", UINT64_FORMAT);
+			f[4] = '*';
+			f[strlen(f)-1] = 'x';
+			smiAsprintf(&s, f, ss, xlen, vv);
+			smiFree(ss);
+			break;
+		    case 'b':
+			k = pfx * 8 - 1;
+			if (k > sizeof(SmiUnsigned64) * 8 - 1)
+			    k = sizeof(SmiUnsigned64) * 8 - 1;
+			for (j = 0; k >= 0; k--, j++) {
+			    ss = s;
+			    smiAsprintf(&s, "%s%c",
+					ss, vv & (1 << k) ? '1' : '0');
+			    smiFree(ss);
+			}
+			break;
+		    }
+		    break;
+		default:
+		    smiFree(s);
+		    if (flags & SMI_RENDER_UNKNOWN) {
+			smiAsprintf(&s, SMI_LANGUAGE_UNKNOWN);
+		    } else {
+			s = NULL;
+		    }
+		    return s;
+		}
+		fmt++;
+
+		/*
+		 * Check for a separator and repeat with last format if
+		 * data is still available.
+		 */
+		if (*fmt && ! isdigit((int) *fmt) && *fmt != '*') {
+		    if (i < smiValuePtr->len) {
+			ss = s;
+			smiAsprintf(&s, "%s%c", ss, fmt[0]);
+			smiFree(ss);
+		    }
+		    fmt++;
+		}
+
+		if (! *fmt && (i < smiValuePtr->len)) {
+		    fmt = last_fmt;
+		}
+	    }
+	}
+	break;
+    case SMI_BASETYPE_ENUM:
+	if ((flags & SMI_RENDER_NAME) && (smiTypePtr)) {
+	    for (nn = smiGetFirstNamedNumber(smiTypePtr); nn;
+		 nn = smiGetNextNamedNumber(nn)) {
+		if (nn->value.value.integer32 == smiValuePtr->value.integer32)
+		    break;
+	    }
+	    if (nn) {
+		if (flags & SMI_RENDER_NUMERIC) {
+		    smiAsprintf(&s, "%s(%ld)",
+				nn->name, nn->value.value.integer32);
+		} else {
+		    smiAsprintf(&s, "%s", nn->name);
+		}
+	    } else {
+		smiAsprintf(&s, "%ld", smiValuePtr->value.integer32);
+	    }
+	} else {
+	    smiAsprintf(&s, "%ld", smiValuePtr->value.integer32);
+	}
+	break;
+    case SMI_BASETYPE_BITS:
+	smiAsprintf(&s, "");
+	for (i = 0, nn = NULL; i < smiValuePtr->len * 8; i++) {
+	    if (smiValuePtr->value.ptr[i/8] & (1 << (7-(i%8)))) {
+		if ((flags & SMI_RENDER_NAME) && (smiTypePtr)) {
+		    for (nn = smiGetFirstNamedNumber(smiTypePtr); nn;
+			 nn = smiGetNextNamedNumber(nn)) {
+			if (nn->value.value.unsigned32 == i)
+			    break;
+		    }
+		}
+		ss = s;
+		if ((flags & SMI_RENDER_NAME) &&
+		    (flags & SMI_RENDER_NUMERIC) && nn) {
+		    smiAsprintf(&s, "%s%s%s(%d)",
+				ss, strlen(ss) ? " " : "", nn->name, i);
+		} else if (nn) {
+		    smiAsprintf(&s, "%s%s%s",
+				ss, strlen(ss) ? " " : "", nn->name);
+		} else {
+		    smiAsprintf(&s, "%s%s%d",
+				ss, strlen(ss) ? " " : "", i);
+		}
+		smiFree(ss);
+	    }
+	}
+	break;
+    case SMI_BASETYPE_FLOAT32:
+    case SMI_BASETYPE_FLOAT64:
+    case SMI_BASETYPE_FLOAT128:
+    case SMI_BASETYPE_UNKNOWN:
+    default:
+	if (flags & SMI_RENDER_UNKNOWN) {
+	    smiAsprintf(&s, SMI_LANGUAGE_UNKNOWN);
+	} else {
+	    s = NULL;
+	}
+	break;
+    }
+
+    return s;
+}
+
+char *smiRenderNode(SmiNode *smiNodePtr, int flags)
+{
+    char *s;
+    SmiModule *modulePtr;
+    
+    if ((!smiNodePtr) || (smiNodePtr->name == NULL)) {
+	if (flags & SMI_RENDER_UNKNOWN) {
+	    smiAsprintf(&s, SMI_UNKNOWN_LABEL);
+	} else {
+	    s = NULL;
+	}
+    } else {
+	modulePtr = smiGetNodeModule(smiNodePtr);
+	if ((!(flags & SMI_RENDER_QUALIFIED)) ||
+	    (!modulePtr) ||
+	    (!strlen(modulePtr->name))) {
+	    smiAsprintf(&s, "%s", smiNodePtr->name);
+	} else {
+	    smiAsprintf(&s, "%s::%s", modulePtr->name, smiNodePtr->name);
+	}
+    }
+    return s;
+}
+
+
+
+char *smiRenderType(SmiType *smiTypePtr, int flags)
+{
+    char *s;
+    SmiModule *modulePtr;
+    
+    if ((!smiTypePtr) || (smiTypePtr->name == NULL)) {
+	if (flags & SMI_RENDER_UNKNOWN) {
+	    smiAsprintf(&s, SMI_UNKNOWN_LABEL);
+	} else {
+	    s = NULL;
+	}
+    } else {
+	modulePtr = smiGetTypeModule(smiTypePtr);
+	if ((!(flags & SMI_RENDER_QUALIFIED)) ||
+	    (!modulePtr) ||
+	    (!strlen(modulePtr->name))) {
+	    smiAsprintf(&s, "%s", smiTypePtr->name);
+	} else {
+	    smiAsprintf(&s, "%s::%s", modulePtr->name, smiTypePtr->name);
+	}
+    }
+    return s;
+}
+
+
+
+int smiAsprintf(char **strp, const char *format, ...)
+{
+    int rc;
+    va_list ap;
+
+    va_start(ap, format);
+    rc = vasprintf(strp, format, ap);
+    va_end(ap);
+    if (! strp) {
+	smiPrintError(NULL, ERR_OUT_OF_MEMORY);
+    }
+    return rc;
+}
+
+
+
+int smiVasprintf(char **strp, const char *format, va_list ap)
+{
+    int rc;
+
+    rc = vasprintf(strp, format, ap);
+    if (! strp) {
+	smiPrintError(NULL, ERR_OUT_OF_MEMORY);
+    }
+    return rc;
+}
+
