@@ -1,30 +1,36 @@
 /*
  * smi.h --
  *
- *      Interface Definition of libsmi (version 2:19:0).
+ *      Interface Definition of libsmi (version 2:23:0).
  *
  * Copyright (c) 1999,2000 Frank Strauss, Technical University of Braunschweig.
  *
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: smi.h.in,v 1.33 2001/10/08 14:16:57 schoenw Exp $
+ * @(#) $Id: smi.h.in 1564 2003-05-19 13:42:35Z strauss $
  */
 
 #ifndef _SMI_H
 #define _SMI_H
 
 #include <stdlib.h>
+#include <stdarg.h>
 #include <time.h>
 
 
-#define SMI_LIBRARY_VERSION "2:19:0"
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
+#define SMI_LIBRARY_VERSION "2:23:0"
 extern const char *smi_library_version;
 
 #define SMI_VERSION_MAJOR 0
-#define SMI_VERSION_MINOR 3
-#define SMI_VERSION_PATCHLEVEL 1
-#define SMI_VERSION_STRING "0.3.1"
+#define SMI_VERSION_MINOR 4
+#define SMI_VERSION_PATCHLEVEL 3
+#define SMI_VERSION_STRING "0.4.3"
 extern const char *smi_version_string;
 
 
@@ -62,7 +68,8 @@ typedef enum SmiLanguage {
     SMI_LANGUAGE_UNKNOWN                = 0,  /* should not occur            */
     SMI_LANGUAGE_SMIV1                  = 1,
     SMI_LANGUAGE_SMIV2                  = 2,
-    SMI_LANGUAGE_SMING                  = 3
+    SMI_LANGUAGE_SMING                  = 3,
+    SMI_LANGUAGE_SPPI                   = 4
 } SmiLanguage;
 
 /* SmiBasetype -- base types of all languages                                */
@@ -72,23 +79,23 @@ typedef enum SmiBasetype {
     SMI_BASETYPE_OCTETSTRING            = 2,
     SMI_BASETYPE_OBJECTIDENTIFIER       = 3,
     SMI_BASETYPE_UNSIGNED32             = 4,
-    SMI_BASETYPE_INTEGER64              = 5,  /* only SMIng                  */
-    SMI_BASETYPE_UNSIGNED64             = 6,  /* SMIv2 and SMIng             */
+    SMI_BASETYPE_INTEGER64              = 5,  /* SMIng and SPPI              */
+    SMI_BASETYPE_UNSIGNED64             = 6,  /* SMIv2, SMIng and SPPI       */
     SMI_BASETYPE_FLOAT32                = 7,  /* only SMIng                  */
     SMI_BASETYPE_FLOAT64                = 8,  /* only SMIng                  */
     SMI_BASETYPE_FLOAT128               = 9,  /* only SMIng                  */
     SMI_BASETYPE_ENUM                   = 10,
-    SMI_BASETYPE_BITS                   = 11  /* only SMIv2 and SMIng        */
+    SMI_BASETYPE_BITS                   = 11  /* SMIv2, SMIng and SPPI       */
 } SmiBasetype;
 
 /* SmiStatus -- values of status levels                                      */
 typedef enum SmiStatus {
     SMI_STATUS_UNKNOWN          = 0, /* should not occur                     */
-    SMI_STATUS_CURRENT          = 1, /* only SMIv2 and SMIng                 */
-    SMI_STATUS_DEPRECATED       = 2, /* SMIv1, SMIv2 and SMIng               */
+    SMI_STATUS_CURRENT          = 1, /* only SMIv2, SMIng and SPPI           */
+    SMI_STATUS_DEPRECATED       = 2, /* SMIv1, SMIv2, SMIng and SPPI         */
     SMI_STATUS_MANDATORY        = 3, /* only SMIv1                           */
     SMI_STATUS_OPTIONAL         = 4, /* only SMIv1                           */
-    SMI_STATUS_OBSOLETE         = 5  /* SMIv1, SMIv2 and SMIng               */
+    SMI_STATUS_OBSOLETE         = 5  /* SMIv1, SMIv2, SMIng and SPPI         */
 } SmiStatus;
 
 /* SmiAccess -- values of access levels                                      */
@@ -98,7 +105,10 @@ typedef enum SmiAccess {
     SMI_ACCESS_NOT_ACCESSIBLE   = 2, /* the values 2 to 5 are allowed to be  */
     SMI_ACCESS_NOTIFY           = 3, /* compared by relational operators.    */
     SMI_ACCESS_READ_ONLY        = 4,
-    SMI_ACCESS_READ_WRITE       = 5
+    SMI_ACCESS_READ_WRITE       = 5,
+    SMI_ACCESS_INSTALL          = 6, /* these three entries are only valid   */
+    SMI_ACCESS_INSTALL_NOTIFY   = 7, /* for SPPI                             */
+    SMI_ACCESS_REPORT_ONLY      = 8
 } SmiAccess;
 
 /* SmiNodekind -- type or statement that leads to a definition               */
@@ -133,6 +143,9 @@ typedef enum SmiDecl {
     SMI_DECL_MODULECOMPLIANCE   = 13,
     SMI_DECL_AGENTCAPABILITIES  = 14,
     SMI_DECL_TEXTUALCONVENTION  = 15,
+    SMI_DECL_MACRO	        = 16,
+    SMI_DECL_COMPL_GROUP        = 17,
+    SMI_DECL_COMPL_OBJECT       = 18,
     /* SMIng statements */
     SMI_DECL_MODULE             = 33,
     SMI_DECL_EXTENSION          = 34,
@@ -393,6 +406,7 @@ extern SmiOption *smiGetNextOption(SmiOption *smiOptionPtr);
 
 extern SmiNode *smiGetOptionNode(SmiOption *smiOptionPtr);
 
+extern int smiGetOptionLine(SmiOption *smiOptionPtr);
 
 
 extern SmiRefinement *smiGetFirstRefinement(SmiNode *smiComplianceNodePtr);
@@ -405,6 +419,46 @@ extern SmiType *smiGetRefinementType(SmiRefinement *smiRefinementPtr);
 
 extern SmiType *smiGetRefinementWriteType(SmiRefinement *smiRefinementPtr);
 
+extern int smiGetRefinementLine(SmiRefinement *smiRefinementPtr);
+
+
+extern SmiElement *smiGetFirstUniquenessElement(SmiNode *smiNodePtr);
+
+#define smiGetNextUniquenessElement(p) smiGetNextElement(p)
+
+extern char *smiRenderOID(unsigned int oidlen, SmiSubid *oid, int flags);
+
+extern char *smiRenderValue(SmiValue *smiValuePtr, SmiType *smiTypePtr,
+			    int flags);
+
+extern char *smiRenderNode(SmiNode *smiNodePtr, int flags);
+
+extern char *smiRenderType(SmiType *smiTypePtr, int flags);
+
+#define SMI_RENDER_NUMERIC   0x01 /* render as numeric values */
+#define SMI_RENDER_NAME      0x02 /* render as names */
+#define SMI_RENDER_QUALIFIED 0x04 /* render names with module prefix */
+#define SMI_RENDER_FORMAT    0x08 /* render by applying the type's format if
+				     type is given and format is present */
+#define SMI_RENDER_PRINTABLE 0x10 /* render string values as a printable
+				     string if all octets are isprint() */
+#define SMI_RENDER_UNKNOWN   0x20 /* render even unknown items as strings
+  				     ("<unknown>") so that we never get NULL */
+#define SMI_RENDER_ALL       0xff /* render as `human friendly' as possible */
+
+#define SMI_UNKNOWN_LABEL "<unknown>"
+
+   
+
+extern int smiAsprintf(char **strp, const char *format, ...);
+
+extern int smiVasprintf(char **strp, const char *format, va_list ap);
+
+
+
+#ifdef __cplusplus
+}
+#endif
 
 
 #endif /* _SMI_H */
