@@ -12,7 +12,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: dump-corba.c,v 1.45 2002/10/30 09:17:37 schoenw Exp $
+ * @(#) $Id: dump-corba.c 1656 2004-07-21 08:00:25Z schoenw $
  */
 
 #include <config.h>
@@ -319,13 +319,15 @@ static int current(SmiStatus status)
 
 static char *getTimeString(time_t t)
 {
-    static char   s[27];
+    static char   *s = NULL;
     struct tm	  *tm;
 
+    if (s) xfree(s);
+
     tm = gmtime(&t);
-    sprintf(s, "%04d%02d%02d%02d%02dZ",
-	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-	    tm->tm_hour, tm->tm_min);
+    smiAsprintf(&s, "%04d%02d%02d%02d%02dZ",
+		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+		tm->tm_hour, tm->tm_min);
     return s;
 }
 
@@ -671,6 +673,41 @@ static void fprintMultiline(FILE *f, const char *s)
 
 
 
+static char *
+getStringTime(time_t t)
+{
+    static char   s[27];
+    struct tm	  *tm;
+
+    tm = gmtime(&t);
+    sprintf(s, "%04d-%02d-%02d %02d:%02d",
+	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+	    tm->tm_hour, tm->tm_min);
+    return s;
+}
+
+
+
+static void
+fprintCommentString(FILE *f, char *s)
+{
+    int i, len;
+
+    if (s) {
+	fprintf(f, " *   ");
+	len = strlen(s);
+	for (i = 0; i < len; i++) {
+	    fputc(s[i], f);
+	    if (s[i] == '\n') {
+		fprintf(f, " *   ");
+	    }
+	}
+	fputc('\n', f);
+    }
+}
+
+
+
 static char* translate(char *s)
 {
     int i;
@@ -1488,18 +1525,41 @@ static void fprintDisplayHints(FILE *f, SmiModule *smiModule)
 
 static void dumpIdl(SmiModule *smiModule)
 {
-    char      *idlModuleName;
-    FILE      *f;
+    char        *idlModuleName;
+    FILE        *f;
+    SmiRevision *smiRevision;
+    char        *date;
 
     f = createFile(getIdlModuleName(smiModule->name), ".idl");
     if (! f) {
         return;
     }
 
+    fprintf(f,
+	    "/*	\t\t\t\t\t\t-- DO NOT EDIT --\n"
+	    " * Generated by smidump version " SMI_VERSION_STRING ":\n");
 
-    fprint(f, "/*\n * This CORBA IDL file has been generated by smidump "
-	  SMI_VERSION_STRING ". Do not edit.\n */\n\n");
+    fprintf(f, " *   smidump -f corba %s\n *\n", smiModule->name);
+
+    fprintf(f,
+	    " * Derived from %s:\n", smiModule->name);
+    fprintCommentString(f, smiModule->description);
+
+    for (smiRevision = smiGetFirstRevision(smiModule);
+	 smiRevision;
+	 smiRevision = smiGetNextRevision(smiRevision)) {
+	date = getStringTime(smiRevision->date);
+	fprintf(f,
+		" *\n"
+		" * Revision %s:\n", date);
+	fprintCommentString(f, smiRevision->description);
+    }
     
+    fprintf(f,
+	    " *\n * $I" "d$\n"
+	    " */\n"
+	    "\n");
+
     idlModuleName = getIdlModuleName(smiModule->name);
     createImportList(smiModule);
 
