@@ -10,7 +10,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: dump-python.c,v 1.12 2002/10/30 09:17:37 schoenw Exp $
+ * @(#) $Id: dump-python.c 1645 2004-06-07 22:01:04Z schoenw $
  */
 
 /*
@@ -39,6 +39,8 @@
 #define  INDENTMAX	64   /* max column to fill, break lines otherwise */
 
 
+static int silent = 0;
+
 
 typedef struct PythonEscape {
     char character;
@@ -645,7 +647,8 @@ static void fprintTypedef(FILE *f, int indent, SmiType *smiType)
     
     parentType = smiGetParentType(smiType);
     parentModule = smiGetTypeModule(parentType);
-    if (parentType && parentModule && strlen(parentModule->name)) {
+    if (parentType && parentType->name &&
+	parentModule && strlen(parentModule->name)) {
 	fprintSegment(f, indent + INDENT, "\"parent module\" : {\n", 0);
 	fprintSegment(f, indent + (2 * INDENT), "", 0);
 	fprint(f, "\"name\" : \"%s\",\n", parentModule->name);
@@ -1010,11 +1013,41 @@ static void fprintCompliances(FILE *f, SmiModule *smiModule)
 
 
 
+static void dumpPythonModule(FILE *f, SmiModule *smiModule)
+{
+    if (! silent) {
+	fprint(f, "# python version 1.0\t\t\t\t\t\tDO NOT EDIT\n#\n");
+	fprint(f, "# Generated by smidump version "
+	       SMI_VERSION_STRING ":\n#\n");
+	fprintf(f, "#   smidump -f python %s\n\n", smiModule->name);
+    }
+    
+    fprint(f, "FILENAME = \"%s\"\n\n", smiModule->path);
+    
+    fprint(f, "MIB = {\n");
+    fprintSegment(f, INDENT, "", 0);
+    fprint(f, "\"moduleName\" : \"%s\",\n\n", smiModule->name);
+    
+    fprintModule(f, smiModule);
+    fprintImports(f, smiModule);
+    fprintTypedefs(f, smiModule);
+    fprintNodes(f, smiModule);
+    fprintNotifications(f, smiModule);
+    fprintGroups(f, smiModule);
+    fprintCompliances(f, smiModule);
+    
+    fprint(f, "}\n");
+}
+
+
+
 static void dumpPython(int modc, SmiModule **modv, int flags, char *output)
 {
     int  i;
     FILE *f = stdout;
     
+    silent = (flags & SMIDUMP_FLAG_SILENT);
+
     if (output) {
 	f = fopen(output, "w");
 	if (!f) {
@@ -1025,25 +1058,7 @@ static void dumpPython(int modc, SmiModule **modv, int flags, char *output)
     }
 
     for (i = 0; i < modc; i++) {
-	fprint(f, "# python version 1.0\n\n");
-	fprint(f, "# This module has been generated by smidump "
-	       SMI_VERSION_STRING ". Do not edit.\n\n");
-	
-	fprint(f, "FILENAME = \"%s\"\n\n", modv[i]->path);
-	
-	fprint(f, "MIB = {\n");
-	fprintSegment(f, INDENT, "", 0);
-	fprint(f, "\"moduleName\" : \"%s\",\n\n", modv[i]->name);
-	
-	fprintModule(f, modv[i]);
-	fprintImports(f, modv[i]);
-	fprintTypedefs(f, modv[i]);
-	fprintNodes(f, modv[i]);
-	fprintNotifications(f, modv[i]);
-	fprintGroups(f, modv[i]);
-	fprintCompliances(f, modv[i]);
-	
-	fprint(f, "}\n");
+	dumpPythonModule(f, modv[i]);
     }
 
     if (fflush(f) || ferror(f)) {
