@@ -8,7 +8,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: dump-sming.c,v 1.91 2002/01/09 12:02:03 schoenw Exp $
+ * @(#) $Id: dump-sming.c 1477 2002-11-19 12:45:41Z strauss $
  */
 
 #include <config.h>
@@ -361,6 +361,9 @@ static void createImportList(SmiModule *smiModule)
 	 smiNode = smiGetNextNode(smiNode, kind)) {
 
 	smiType = smiGetNodeType(smiNode);
+	if (smiType && (smiType->decl == SMI_DECL_IMPLICIT_TYPE)) {
+	    smiType = smiGetParentType(smiType);
+	}
 	if (smiType) {
 	    smiTypeModule = smiGetTypeModule(smiType);
 	    if (smiTypeModule &&
@@ -1307,6 +1310,11 @@ static void dumpSming(int modc, SmiModule **modv, int flags, char *output)
 	
 	fprint(f, "}; // end of module %s.\n", smiModule->name);
     }
+    
+    if (fflush(f) || ferror(f)) {
+	perror("smidump: write error");
+	exit(1);
+    }
 
     if (output) {
 	fclose(f);
