@@ -10,7 +10,14 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: smidiff.c,v 1.36 2002/01/25 09:15:30 schoenw Exp $ 
+ * @(#) $Id: smidiff.c 1654 2004-07-21 07:19:14Z schoenw $ 
+ */
+
+/*
+ * TODO:
+ *
+ * - replacing an implicit type with a named type should cause a real
+ *   error if the new type is formally not identical (Mike Heard)
  */
 
 #include <config.h>
@@ -42,10 +49,11 @@ static const char *newTag = "smidiff:new";
 
 
 typedef struct Error {
-    int level;	/* error level - roughly the same as smilint */
-    int id;	/* error id used in the error() invocation */
-    char *tag;	/* tag for error identification on cmd line */
-    char *fmt;	/* the complete error format string */
+    int level;		/* error level - roughly the same as smilint */
+    int id;		/* error id used in the error() invocation */
+    char *tag;		/* tag for error identification on cmd line */
+    char *fmt;		/* the complete error format string */
+    char *description;	/* description of the error message */
 } Error;
 
 
@@ -129,173 +137,245 @@ typedef struct Error {
 #define ERR_LENGTH_OF_TYPE_REMOVED		78
 #define ERR_STATUS_ADDED			79
 #define ERR_STATUS_REMOVED			80
+#define ERR_MANDATORY_GROUP_ADDED		81
+#define ERR_MANDATORY_GROUP_REMOVED		82
+#define ERR_MANDATORY_EXT_GROUP_ADDED		83
+#define ERR_MANDATORY_EXT_GROUP_REMOVED		84
+#define ERR_OPTION_ADDED			85
+#define ERR_OPTION_REMOVED			86
+#define ERR_EXT_OPTION_ADDED			87
+#define ERR_EXT_OPTION_REMOVED			88
+#define ERR_REFINEMENT_ADDED			89
+#define ERR_REFINEMENT_REMOVED			90
+#define ERR_EXT_REFINEMENT_ADDED		91
+#define ERR_EXT_REFINEMENT_REMOVED		92
 
 static Error errors[] = {
     { 0, ERR_INTERNAL, "internal", 
-      "internal error!!!" },
+      "internal error!!!", NULL },
     { 1, ERR_TYPE_REMOVED, "type-removed",
-      "type `%s' has been deleted" },
+      "type `%s' has been deleted", NULL },
     { 5, ERR_TYPE_ADDED, "type-added",
-      "type `%s' has been added" },
+      "type `%s' has been added", NULL },
     { 1, ERR_NODE_REMOVED, "node-removed",
-      "%s `%s' has been deleted" },
+      "%s `%s' has been deleted", NULL },
     { 5, ERR_NODE_ADDED, "node-added",
-      "%s `%s' has been added" },
+      "%s `%s' has been added", NULL },
     { 1, ERR_BASETYPE_CHANGED, "basetype-changed",
-      "base type of `%s' changed" },
+      "base type of `%s' changed", NULL },
     { 5, ERR_DECL_CHANGED, "decl-changed",
-      "declaration changed for `%s'" },
+      "declaration changed for `%s'", NULL },
     { 5, ERR_LEGAL_STATUS_CHANGED, "status-change",
-      "legal status change from `%s' to `%s' for `%s'" },
+      "legal status change from `%s' to `%s' for `%s'", NULL },
     { 6, ERR_PREVIOUS_DEFINITION, "previous-definition",
-      "previous definition of `%s'" },
+      "previous definition of `%s'", NULL },
     { 2, ERR_STATUS_CHANGED, "status-change",
-      "status change from `%s' to `%s' for `%s'" },
+      "status change from `%s' to `%s' for `%s'", NULL },
     { 5, ERR_DESCR_ADDED, "description-added",
-      "description added to `%s'" },
+      "description added to `%s'", NULL },
     { 2, ERR_DESCR_REMOVED, "description-removed",
-      "description removed from `%s'" },
+      "description removed from `%s'", NULL },
     { 5, ERR_DESCR_CHANGED, "description-changed",
-      "description of `%s' changed" },
+      "description of %s `%s' changed", NULL },
     { 5, ERR_REF_ADDED, "ref-added",
-      "reference added to `%s'" },
+      "reference added to `%s'", NULL },
     { 3, ERR_REF_REMOVED, "ref-removed",
-      "reference removed from `%s'" },
+      "reference removed from `%s'", NULL },
     { 5, ERR_REF_CHANGED, "ref-changed",
-      "reference of `%s' changed" },
+      "reference of `%s' changed", NULL },
     { 5, ERR_FORMAT_ADDED, "format-added",
-      "format added to `%s'" },
+      "format added to `%s'", NULL },
     { 3, ERR_FORMAT_REMOVED, "format-removed",
-      "format removed from `%s'" },
+      "format removed from `%s'", NULL },
     { 5, ERR_FORMAT_CHANGED, "format-changed",
-      "format of `%s' changed" },
+      "format of `%s' changed", NULL },
     { 5, ERR_UNITS_ADDED, "units-added",
-      "units added to `%s'" },
+      "units added to `%s'", NULL },
     { 3, ERR_UNITS_REMOVED, "units-removed",
-      "units removed from `%s'" },
+      "units removed from `%s'", NULL },
     { 5, ERR_UNITS_CHANGED, "units-changed",
-      "units of `%s' changed" },
+      "units of `%s' changed", NULL },
     { 5, ERR_ACCESS_ADDED, "access-added",
-      "access added to `%s'" },
+      "access added to `%s'", NULL },
     { 3, ERR_ACCESS_REMOVED, "access-removed",
-      "access removed from `%s'" },
+      "access removed from `%s'", NULL },
     { 5, ERR_ACCESS_CHANGED, "access-changed",
-      "access of `%s' changed from `%s' to `%s'" },
+      "access of `%s' changed from `%s' to `%s'", NULL },
     { 5, ERR_NAME_ADDED, "name-added",
-      "name added to `%s'" },
+      "name added to `%s'", NULL },
     { 3, ERR_NAME_REMOVED, "name-removed",
-      "name removed from `%s'" },
+      "name removed from `%s'", NULL },
     { 5, ERR_NAME_CHANGED, "name-changed",
-      "name changed from `%s' to `%s'" },
+      "name changed from `%s' to `%s'", NULL },
     { 3, ERR_TO_IMPLICIT, "to-implicit",
-      "implicit type for `%s' replaces type `%s'" },
-    { 3, ERR_FROM_IMPLICIT, "from-implicit",
-      "type `%s' replaces implicit type for `%s'" },
+      "implicit type for `%s' replaces type `%s'", NULL },
+    { 5, ERR_FROM_IMPLICIT, "from-implicit",
+      "type `%s' replaces implicit type for `%s'", NULL },
     { 3, ERR_RANGE_ADDED, "range-added",
-      "range `%s' added to type used in `%s'" },
+      "range `%s' added to type used in `%s'", NULL },
     { 3, ERR_RANGE_REMOVED, "range-removed",
-      "range `%s' removed from type used in `%s'" },
+      "range `%s' removed from type used in `%s'", NULL },
     { 3, ERR_RANGE_CHANGED, "range-changed",
-      "range of type used in `%s' changed from `%s' to `%s'" }, 
+      "range of type used in `%s' changed from `%s' to `%s'", NULL }, 
     { 3, ERR_DEFVAL_ADDED, "defval-added",
-      "default value added to `%s'" },
+      "default value added to `%s'", NULL },
     { 3, ERR_DEFVAL_REMOVED, "defval-removed",
-      "default value removed from `%s'" },
+      "default value removed from `%s'", NULL },
     { 3, ERR_DEFVAL_CHANGED, "defval-changed",
-      "default value of `%s' changed" },
-    { 3, ERR_ORGA_ADDED, "organization-added",
-      "organization added to `%s'" },
+      "default value of `%s' changed", NULL },
+    { 5, ERR_ORGA_ADDED, "organization-added",
+      "organization added to `%s'", NULL },
     { 3, ERR_ORGA_REMOVED, "organization-removed",
-      "organization removed from `%s'" },
-    { 3, ERR_ORGA_CHANGED, "organization-changed",
-      "organization of `%s' changed" },
-    { 3, ERR_CONTACT_ADDED, "contact-added",
-      "contact added to `%s'" },
+      "organization removed from `%s'", NULL },
+    { 5, ERR_ORGA_CHANGED, "organization-changed",
+      "organization of `%s' changed", NULL },
+    { 5, ERR_CONTACT_ADDED, "contact-added",
+      "contact added to `%s'", NULL },
     { 3, ERR_CONTACT_REMOVED, "contact-removed",
-      "contact removed from `%s'" },
-    { 3, ERR_CONTACT_CHANGED, "contact-changed",
-      "contact of `%s' changed" },
+      "contact removed from `%s'", NULL },
+    { 5, ERR_CONTACT_CHANGED, "contact-changed",
+      "contact of `%s' changed", NULL },
     { 3, ERR_SMIVERSION_CHANGED, "smi-version-changed",
-      "SMI version changed" },
+      "SMI version changed", NULL },
     { 5, ERR_REVISION_ADDED, "revision-added",
-      "revision `%s' added" },
+      "revision `%s' added", NULL },
     { 3, ERR_REVISION_REMOVED, "revision-removed",
-      "revision `%s' removed" },
-    { 3, ERR_REVISION_CHANGED, "revision-changed",
-      "revision `%s' changed" },
+      "revision `%s' removed", NULL },
+    { 5, ERR_REVISION_CHANGED, "revision-changed",
+      "revision `%s' changed", NULL },
     { 3, ERR_LENGTH_CHANGED, "range-changed",
-      "size of type used in `%s' changed from `%s' to `%s'" },
+      "size of type used in `%s' changed from `%s' to `%s'", NULL },
     { 3, ERR_LENGTH_OF_TYPE_CHANGED, "range-changed",
-      "size of type `%s' changed from `%s' to `%s'" },
+      "size of type `%s' changed from `%s' to `%s'", NULL },
     { 3, ERR_LENGTH_ADDED, "range-added",
-      "size `%s' added to type used in `%s'" },
+      "size `%s' added to type used in `%s'", NULL },
     { 2, ERR_MEMBER_ADDED, "member-added",
-      "member `%s' added to group `%s'" },
+      "member `%s' added to group `%s'", NULL },
     { 2, ERR_MEMBER_REMOVED, "member-removed",
-      "member `%s' removed from group `%s'" },
+      "member `%s' removed from group `%s'", NULL },
     { 3, ERR_MEMBER_CHANGED, "member-changed",
-      "member `%s' changed in group `%s'" },
+      "member `%s' changed in group `%s'", NULL },
     { 3, ERR_OBJECT_ADDED, "object-added",
-      "object `%s' added" },
+      "object `%s' added", NULL },
     { 2, ERR_OBJECT_REMOVED, "object-removed",
-      "object `%s' removed" },
+      "object `%s' removed", NULL },
     { 3, ERR_OBJECT_CHANGED, "object-changed",
-      "object `%s' changed" },
+      "object `%s' changed", NULL },
     { 5, ERR_NAMED_NUMBER_ADDED, "named-number-added",
-      "named number `%s' added to type used in `%s'" },
+      "named number `%s' added to type used in `%s'", NULL },
     { 2, ERR_NAMED_NUMBER_REMOVED, "named-number-removed",
-      "named number `%s' removed from type used in `%s'" },
+      "named number `%s' removed from type used in `%s'", NULL },
     { 5, ERR_NAMED_NUMBER_CHANGED, "named-number-changed",
-      "named number `%s' changed to `%s' at type used in `%s'" },
+      "named number `%s' changed to `%s' at type used in `%s'", NULL },
     { 3, ERR_NAMED_BIT_ADDED_OLD_BYTE, "named-bit-added-old-byte",
-      "named bit `%s' added without starting in a new byte in type used in `%s'" },
+      "named bit `%s' added without starting in a new byte in type used in `%s'", NULL },
      { 3, ERR_LENGTH_REMOVED, "range-removed",
-      "size `%s' removed from type used in `%s'" },
+      "size `%s' removed from type used in `%s'", NULL },
     { 2, ERR_NODEKIND_CHANGED, "nodekind-changed",
-      "node kind of `%s' changed" },
+      "node kind of `%s' changed", NULL },
     { 2, ERR_INDEXKIND_CHANGED, "indexkind-changed",
-      "changed kind of index from `%s' to `%s' in node `%s'" },
+      "changed kind of index from `%s' to `%s' in node `%s'", NULL },
     { 2, ERR_INDEX_CHANGED, "index-changed",
-      "index of `%s' changed from %s to %s" },
-    { 6, ERR_TYPE_IS_AND_WAS, "type-is-and-was",
-      "type changed from %s to %s" },
+      "index of `%s' changed from %s to %s", NULL },
+    { 5, ERR_TYPE_IS_AND_WAS, "type-is-and-was",
+      "type changed from %s to %s", NULL },
     { 3, ERR_RANGE_OF_TYPE_CHANGED, "range-changed",
-      "range of type `%s' changed from `%s' to `%s'" },
+      "range of type `%s' changed from `%s' to `%s'", NULL },
     { 3, ERR_RANGE_OF_TYPE_ADDED, "range-added",
-      "range `%s' added to type `%s'" },
+      "range `%s' added to type `%s'", NULL },
     { 3, ERR_RANGE_OF_TYPE_REMOVED, "range-removed",
-      "range `%s' removed from type `%s'" },
+      "range `%s' removed from type `%s'", NULL },
     { 6, ERR_TYPE_BASED_ON, "type-based-on",
-      "type `%s' based on `%s'" },
+      "type `%s' based on `%s'", NULL },
     { 2, ERR_INDEX_AUGMENT_CHANGED, "index-changed",
-      "index of `%s' changed from augmenting `%s' to augmenting `%s'" },
+      "index of `%s' changed from augmenting `%s' to augmenting `%s'", NULL },
     { 2, ERR_NAMED_NUMBER_OF_TYPE_REMOVED, "named-number-removed",
-      "named number `%s' removed from type `%s'" },
+      "named number `%s' removed from type `%s'", NULL },
     { 5, ERR_NAMED_NUMBER_TO_TYPE_ADDED, "named-number-added",
-      "named number `%s' added to type `%s'" },
+      "named number `%s' added to type `%s'", NULL },
     { 5, ERR_NAMED_NUMBER_OF_TYPE_CHANGED, "named-number-changed",
-      "named number `%s' changed to `%s' in type `%s'" },
+      "named number `%s' changed to `%s' in type `%s'", NULL },
     { 3, ERR_NAMED_BIT_OF_TYPE_ADDED_OLD_BYTE, "named-bit-added-old-byte",
-      "named bit `%s' added without starting in a new byte in type `%s'" },
+      "named bit `%s' added without starting in a new byte in type `%s'", NULL },
     { 6, ERR_PREVIOUS_IMPLICIT_DEFINITION, "previous-definition",
-      "previous implicit definition" },
+      "previous implicit definition", NULL },
     { 2, ERR_STATUS_CHANGED_IMPLICIT, "status-change",
-      "status change from `%s' to `%s' for implicit type" },
+      "status change from `%s' to `%s' for implicit type", NULL },
     { 5, ERR_LEGAL_STATUS_CHANGED_IMPLICIT, "status-change",
-      "legal status change from `%s' to `%s' for implicit type" },
+      "legal status change from `%s' to `%s' for implicit type", NULL },
     { 3, ERR_LENGTH_OF_TYPE_ADDED, "range-added",
-      "size `%s' added to type `%s'" },
+      "size `%s' added to type `%s'", NULL },
     { 3, ERR_LENGTH_OF_TYPE_REMOVED, "range-removed",
-      "size `%s' removed from type `%s'" },
+      "size `%s' removed from type `%s'", NULL },
     { 5, ERR_STATUS_ADDED, "status-added",
-      "status added to `%s'" },
+      "status added to `%s'", NULL },
     { 3, ERR_STATUS_REMOVED, "status-removed",
-      "status removed from `%s'" },
+      "status removed from `%s'", NULL },
+    { 2, ERR_MANDATORY_GROUP_ADDED, "mandatory-added",
+      "mandatory group `%s' added to `%s'", NULL },
+    { 2, ERR_MANDATORY_GROUP_REMOVED, "mandatory-removed",
+      "mandatory group `%s' removed from `%s'", NULL },
+    { 2, ERR_MANDATORY_EXT_GROUP_ADDED, "mandatory-added",
+      "mandatory group `%s::%s' added to `%s'", NULL },
+    { 2, ERR_MANDATORY_EXT_GROUP_REMOVED, "mandatory-removed",
+      "mandatory group `%s::%s' removed from `%s'", NULL },
+    { 2, ERR_OPTION_ADDED, "option-added",
+      "optional group `%s' added to `%s'", NULL },
+    { 2, ERR_OPTION_REMOVED, "option-removed",
+      "optional group `%s' removed from `%s'", NULL },
+    { 2, ERR_EXT_OPTION_ADDED, "option-added",
+      "optional group `%s::%s' added to `%s'", NULL },
+    { 2, ERR_EXT_OPTION_REMOVED, "option-removed",
+      "optional group `%s::%s' removed from `%s'", NULL },
+    { 5, ERR_REFINEMENT_ADDED, "refinement-added",
+      "object refinement for `%s' added to `%s'", NULL },
+    { 2, ERR_REFINEMENT_REMOVED, "refinement-removed",
+      "object refinement for `%s' removed from `%s'", NULL },
+    { 5, ERR_EXT_REFINEMENT_ADDED, "refinement-added",
+      "object refinement for `%s::%s' added to `%s'", NULL },
+    { 2, ERR_EXT_REFINEMENT_REMOVED, "refinement-removed",
+      "object refinement for `%s::%s' removed from `%s'", NULL },
     { 0, 0, NULL, NULL }
 };
 
 
 
+static char *smiStringDecl(SmiDecl macro)
+{
+    return
+        (macro == SMI_DECL_UNKNOWN)           ? "unknown construct" :
+        (macro == SMI_DECL_IMPLICIT_TYPE)     ? "implicit construct" :
+        (macro == SMI_DECL_TYPEASSIGNMENT)    ? "type assignment" :
+        (macro == SMI_DECL_IMPL_SEQUENCEOF)   ? "implicit sequence-of construct" :
+        (macro == SMI_DECL_VALUEASSIGNMENT)   ? "value assignment" :
+        (macro == SMI_DECL_OBJECTTYPE)        ? "object definition" :
+        (macro == SMI_DECL_OBJECTIDENTITY)    ? "object identity definition" :
+        (macro == SMI_DECL_MODULEIDENTITY)    ? "module identity definition" :
+        (macro == SMI_DECL_NOTIFICATIONTYPE)  ? "notification definition" :
+        (macro == SMI_DECL_TRAPTYPE)          ? "trap definition" :
+        (macro == SMI_DECL_OBJECTGROUP)       ? "object group definition" :
+        (macro == SMI_DECL_NOTIFICATIONGROUP) ? "notification group definition" :
+        (macro == SMI_DECL_MODULECOMPLIANCE)  ? "module compliance definition" :
+        (macro == SMI_DECL_AGENTCAPABILITIES) ? "agent capabilities definition" :
+        (macro == SMI_DECL_TEXTUALCONVENTION) ? "textual convention definition" :
+        (macro == SMI_DECL_MACRO)             ? "macro definition" :
+        (macro == SMI_DECL_COMPL_GROUP)       ? "optional group" :
+        (macro == SMI_DECL_COMPL_OBJECT)      ? "object refinement" :
+        (macro == SMI_DECL_MODULE)	      ? "module" :
+        (macro == SMI_DECL_TYPEDEF)	      ? "typedef" :
+        (macro == SMI_DECL_NODE)	      ? "node" :
+        (macro == SMI_DECL_SCALAR)	      ? "scalar" :
+        (macro == SMI_DECL_TABLE)	      ? "table" :
+        (macro == SMI_DECL_ROW)		      ? "row" :
+        (macro == SMI_DECL_COLUMN)	      ? "column" :
+        (macro == SMI_DECL_NOTIFICATION)      ? "notification" :
+        (macro == SMI_DECL_GROUP)	      ? "group" :
+        (macro == SMI_DECL_COMPLIANCE)	      ? "compliance" :
+                                                "<UNDEFINED>";
+}
+
+
+
 static void
 setErrorSeverity(char *pattern, int severity)
 {
@@ -334,17 +414,21 @@ printError(SmiModule *smiModule, int id, int line, va_list ap)
 	    fprintf(stdout, ":%d", line);
 	}
 	fprintf(stdout, " ");
-#if 0
-	if (errors[i].level > 3) {
-	    fprintf(stdout, "warning: ");
-	}
-#endif
 	if (sFlag) {
 	    fprintf(stdout, "[%d] ", errors[i].level);
 	}
 	if (mFlag) {
 	    fprintf(stdout, "{%s} ", errors[i].tag);
 	}
+	switch (errors[i].level) {
+	case 4:
+	case 5:
+	    fprintf(stdout, "warning: ");
+	    break;
+	case 6:	
+	    fprintf(stdout, "info: ");
+	    break;
+	}
 	vfprintf(stdout, errors[i].fmt, ap);
 	fprintf(stdout, "\n");
     }
@@ -530,15 +614,15 @@ checkStatus(SmiModule *oldModule, int oldLine,
     } else {
 	if (name) {
 	    printErrorAtLine(newModule, ERR_STATUS_CHANGED, newLine,
-			     getStringStatus( oldStatus ),
-			     getStringStatus( newStatus ),
+			     getStringStatus(oldStatus),
+			     getStringStatus(newStatus),
 			     name);
 	    code |= CODE_SHOW_PREVIOUS;
 	} else {
 	    printErrorAtLine(newModule, ERR_STATUS_CHANGED_IMPLICIT,
 			     newLine,
-			     getStringStatus( oldStatus ),
-			     getStringStatus( newStatus ));
+			     getStringStatus(oldStatus),
+			     getStringStatus(newStatus));
 	    code |= CODE_SHOW_PREVIOUS_IMPLICIT;
 	}
     }
@@ -597,7 +681,7 @@ checkAccess(SmiModule *oldModule, int oldLine,
 static int
 checkDescription(SmiModule *oldModule, int oldLine,
 		 SmiModule *newModule, int newLine,
-		 char *name, char *oldDescr, char *newDescr)
+		 char *name, SmiDecl decl, char *oldDescr, char *newDescr)
 {
     int code = 0;
     
@@ -614,7 +698,7 @@ checkDescription(SmiModule *oldModule, int oldLine,
 
     if (oldDescr && newDescr && diffStrings(oldDescr, newDescr)) {
 	printErrorAtLine(newModule, ERR_DESCR_CHANGED,
-			 newLine, name);
+			 newLine, smiStringDecl(decl), name);
 	code |= CODE_SHOW_PREVIOUS;
     }
 
@@ -1143,7 +1227,8 @@ checkDefVal(SmiModule *oldModule, int oldLine,
 {
     if ((oldVal.basetype != SMI_BASETYPE_UNKNOWN) && 
 	(newVal.basetype == SMI_BASETYPE_UNKNOWN)) {
-	printErrorAtLine(oldModule, ERR_DEFVAL_REMOVED, oldLine, name);
+	printErrorAtLine(newModule, ERR_DEFVAL_REMOVED, newLine, name);
+	printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION, oldLine, name);
 	return;
     }
 
@@ -1153,12 +1238,14 @@ checkDefVal(SmiModule *oldModule, int oldLine,
 	return;
     }
 
+#if 0 /* changed base type is reported, anyway. */
     if (oldVal.basetype != newVal.basetype) {
 	printErrorAtLine(newModule, ERR_DEFVAL_CHANGED, newLine, name);
 	printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION, oldLine, name);
 	return;
     }
-	
+#endif
+    
     if (cmpSmiValues(oldVal, newVal)) {
 	printErrorAtLine(newModule, ERR_DEFVAL_CHANGED, newLine,name);
 	printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION, oldLine, name);
@@ -1371,11 +1458,29 @@ checkNamedNumbers(SmiModule *oldModule, int oldLine,
 static void
 checkTypeCompatibility(SmiModule *oldModule, SmiNode *oldNode,
 		       SmiType *oldType,
-		       SmiModule *newModule, SmiNode *newNode,
+		       SmiModule *newModule, int newLine,
 		       SmiType *newType)
 {
-    int oldLine, newLine;
+    int oldLine;
     char *oldName;
+
+    if ((!oldType) && (!newType)) return;
+
+    if (!oldType) {
+	if (newType->name) {
+	    printErrorAtLine(newModule, ERR_TYPE_ADDED,
+			     smiGetTypeLine(newType));
+	}
+	return;
+    }
+    
+    if (!newType) {
+	if (oldType->name) {
+	    printErrorAtLine(oldModule, ERR_TYPE_REMOVED,
+			     smiGetTypeLine(oldType));
+	}
+	return;
+    }
     
     if (oldType->basetype != newType->basetype) {
 	if( newType->name ) {
@@ -1397,23 +1502,66 @@ checkTypeCompatibility(SmiModule *oldModule, SmiNode *oldNode,
     }
 
     oldLine = oldNode ? smiGetNodeLine( oldNode ) : smiGetTypeLine( oldType );
-    newLine = newNode ? smiGetNodeLine( newNode ) : smiGetTypeLine( newType );
-    checkNamedNumbers(oldModule, oldLine,
-		      newModule, newLine,
+    checkNamedNumbers(oldModule,
+		      oldLine,
+		      newModule,
+		      newLine > 0 ? newLine : smiGetTypeLine( newType ),
 		      oldType->name,
 		      oldNode,
 		      oldType,
 		      newType);
 
     oldName = oldNode ? oldNode->name : NULL;
-    checkRanges(oldModule, oldLine,
-		newModule, newLine,
+    checkRanges(oldModule,
+		oldLine,
+		newModule,
+		newLine > 0 ? newLine : smiGetTypeLine( newType ),
 		oldName,
 		oldType,
 		newType);
 }
 
 
+static void
+checkNodeTypeCompatibility(SmiModule *oldModule, SmiNode *oldNode,
+			   SmiModule *newModule, SmiNode *newNode)
+{
+    SmiType *oldType, *newType;
+    
+    const int oldLine = smiGetNodeLine(oldNode);
+    const int newLine = smiGetNodeLine(newNode);
+
+    oldType = smiGetNodeType(oldNode);
+    newType = smiGetNodeType(newNode);
+
+    if ((!oldType) && (!newType)) return;
+
+    if (oldType && newType && oldType->basetype != newType->basetype) {
+	printErrorAtLine(newModule, ERR_BASETYPE_CHANGED,
+			 newLine, newNode->name);
+	printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
+			 oldLine, oldNode->name);
+	return;
+    }
+
+    checkNamedNumbers(oldModule,
+		      oldLine,
+		      newModule,
+		      newLine > 0 ? newLine : smiGetTypeLine(newType),
+		      oldType->name,
+		      oldNode,
+		      oldType,
+		      newType);
+    checkRanges(oldModule,
+		oldLine,
+		newModule,
+		newLine > 0 ? newLine : smiGetTypeLine(newType),
+		oldNode->name,
+		oldType,
+		newType);
+}
+
+
 static void
 checkTypes(SmiModule *oldModule, SmiNode *oldNode, SmiType *oldType,
 	   SmiModule *newModule, SmiNode *newNode, SmiType *newType)
@@ -1425,7 +1573,9 @@ checkTypes(SmiModule *oldModule, SmiNode *oldNode, SmiType *oldType,
 		      oldType->name, newType->name);
 
     checkTypeCompatibility(oldModule, oldNode, oldType,
-			   newModule, newNode, newType);
+			   newModule,
+			   newNode ? smiGetNodeLine(newNode) : smiGetTypeLine(newType),
+			   newType);
     
     checkDefVal(oldModule, smiGetTypeLine(oldType),
 		newModule, smiGetTypeLine(newType),
@@ -1455,7 +1605,7 @@ checkTypes(SmiModule *oldModule, SmiNode *oldNode, SmiType *oldType,
 
     code |= checkDescription(oldModule, smiGetTypeLine(oldType),
 			     newModule, smiGetTypeLine(newType),
-			     newType->name,
+			     newType->name, oldType->decl,
 			     oldType->description, newType->description);
 
     code |= checkReference(oldModule, smiGetTypeLine(oldType),
@@ -1716,23 +1866,18 @@ checkObject(SmiModule *oldModule, SmiNode *oldNode,
 			     newNode->name, oldType->name);
 	    printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
 			     smiGetNodeLine(oldNode), oldNode->name);
-	    
-	    checkTypeCompatibility(oldModule, oldNode, oldType,
-				   newModule, newNode, newType);
 	} else if (!oldType->name && newType->name) {
 	    printErrorAtLine(newModule, ERR_FROM_IMPLICIT,
 			     smiGetNodeLine(newNode),
 			     newType->name, oldNode->name);
 	    printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
 			     smiGetNodeLine(oldNode), oldNode->name);
-	    checkTypeCompatibility(oldModule, oldNode, oldType,
-				   newModule, newNode, newType);
-	} else {
-	    checkTypeCompatibility(oldModule, oldNode, oldType,
-				   newModule, newNode, newType);
 	}
     }
 
+    checkNodeTypeCompatibility(oldModule, oldNode,
+			       newModule, newNode);
+
     code |= checkDecl(oldModule, oldLine, newModule, newLine,
 		      newNode->name, oldNode->decl, newNode->decl);
 
@@ -1757,7 +1902,7 @@ checkObject(SmiModule *oldModule, SmiNode *oldNode,
 		       newNode->name, oldNode->units, newNode->units);
 
     code |= checkDescription(oldModule, oldLine, newModule, newLine,
-			     newNode->name,
+			     newNode->name, oldNode->decl,
 			     oldNode->description, newNode->description);
 
     code |= checkReference(oldModule, oldLine, newModule, newLine,
@@ -1916,7 +2061,7 @@ checkNotification(SmiModule *oldModule, const char *oldTag,
 
     code |= checkDescription(oldModule, smiGetNodeLine(oldNode),
 			     newModule, smiGetNodeLine(newNode),
-			     newNode->name,
+			     newNode->name, oldNode->decl,
 			     oldNode->description, newNode->description);
 
     code |= checkReference(oldModule, smiGetNodeLine(oldNode),
@@ -1958,7 +2103,7 @@ diffNotifications(SmiModule *oldModule, const char *oldTag,
 	    && newNode->oidlen == oldNode->oidlen
 	    && smiGetNodeModule(newNode) == newModule) {
 	    checkNotification(oldModule, oldTag, newModule, newTag,
-			      newNode, oldNode);
+			      oldNode, newNode);
 	} else {
 	    printErrorAtLine(oldModule, ERR_NODE_REMOVED,
 			     smiGetNodeLine(oldNode),
@@ -2084,7 +2229,7 @@ diffModules(SmiModule *oldModule, const char *oldTag,
 
     code |= checkDescription(oldModule, oldLine,
 			     newModule, newLine,
-			     newModule->name,
+			     newModule->name, SMI_DECL_MODULEIDENTITY,
 			     oldModule->description, newModule->description);
 
     code |= checkReference(oldModule, oldLine, newModule, newLine, newModule->name,
@@ -2231,7 +2376,7 @@ checkGroup(SmiModule *oldModule, const char *oldTag,
 
     code |= checkDescription(oldModule, smiGetNodeLine(oldNode),
 			     newModule, smiGetNodeLine(newNode),
-			     newNode->name,
+			     newNode->name, oldNode->decl,
 			     oldNode->description, newNode->description);
 
     code |= checkReference(oldModule, smiGetNodeLine(oldNode),
@@ -2268,7 +2413,6 @@ diffGroups(SmiModule *oldModule, const char *oldTag,
     for(oldNode = smiGetFirstNode(oldModule, SMI_NODEKIND_GROUP);
 	oldNode;
         oldNode = smiGetNextNode(oldNode, SMI_NODEKIND_GROUP)) {
-	
 	smiInit(newTag);
 	newNode = smiGetNodeByOID(oldNode->oidlen, oldNode->oid);
 	if (newNode
@@ -2309,6 +2453,359 @@ diffGroups(SmiModule *oldModule, const char *oldTag,
 
 
 
+static void
+checkComplMandatory(SmiModule *oldModule, const char *oldTag,
+		    SmiModule *newModule, const char *newTag,
+		    SmiNode *oldNode, SmiNode *newNode)
+{
+    SmiElement *oldElem, *newElem;
+    SmiNode *oldElemNode, *newElemNode;
+
+    smiInit(oldTag);
+    for (oldElem = smiGetFirstElement(oldNode);
+	 oldElem; oldElem = smiGetNextElement(oldElem)) {
+	oldElemNode = smiGetElementNode(oldElem);
+	smiInit(newTag);
+	for (newElem = smiGetFirstElement(newNode);
+	     newElem; newElem = smiGetNextElement(newElem)) {
+	    newElemNode = smiGetElementNode(newElem);
+	    if (strcmp(oldElemNode->name, newElemNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! newElem) {
+	    if (strcmp(smiGetNodeModule(oldElemNode)->name, oldModule->name)) {
+		printErrorAtLine(oldModule, ERR_MANDATORY_EXT_GROUP_REMOVED,
+				 smiGetNodeLine(oldNode),
+				 oldModule->name, oldElemNode->name,
+				 oldNode->name);
+	    } else {
+		printErrorAtLine(oldModule, ERR_MANDATORY_GROUP_REMOVED,
+				 smiGetNodeLine(oldNode),
+				 oldElemNode->name,
+				 oldNode->name);
+	    }
+	}
+	smiInit(oldTag);
+    }
+
+    smiInit(newTag);
+    for (newElem = smiGetFirstElement(newNode);
+	 newElem; newElem = smiGetNextElement(newElem)) {
+	newElemNode = smiGetElementNode(newElem);
+	smiInit(oldTag);
+	for (oldElem = smiGetFirstElement(oldNode);
+	     oldElem; oldElem = smiGetNextElement(oldElem)) {
+	    oldElemNode = smiGetElementNode(oldElem);
+	    if (strcmp(oldElemNode->name, newElemNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! oldElem) {
+	    if (strcmp(smiGetNodeModule(newElemNode)->name, newModule->name)) {
+		printErrorAtLine(newModule, ERR_MANDATORY_EXT_GROUP_ADDED,
+				 smiGetNodeLine(newNode),
+				 newModule->name, newElemNode->name,
+				 newNode->name);
+	    } else {
+		printErrorAtLine(newModule, ERR_MANDATORY_GROUP_ADDED,
+				 smiGetNodeLine(newNode),
+				 newElemNode->name, newNode->name);
+	    }
+	}
+	smiInit(newTag);
+    }
+}
+
+
+
+static void
+checkComplOptions(SmiModule *oldModule, const char *oldTag,
+		  SmiModule *newModule, const char *newTag,
+		  SmiNode *oldNode, SmiNode *newNode)
+{
+    int code;
+    SmiOption *oldOption, *newOption;
+    SmiNode *oldOptionNode, *newOptionNode;
+
+    smiInit(oldTag);
+    for (oldOption = smiGetFirstOption(oldNode);
+	 oldOption; oldOption = smiGetNextOption(oldOption)) {
+	oldOptionNode = smiGetOptionNode(oldOption);
+	smiInit(newTag);
+	for (newOption = smiGetFirstOption(newNode);
+	     newOption; newOption = smiGetNextOption(newOption)) {
+	    newOptionNode = smiGetOptionNode(newOption);
+	    if (strcmp(oldOptionNode->name, newOptionNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! newOption) {
+	    if (strcmp(smiGetNodeModule(oldOptionNode)->name,
+		       oldModule->name)) {
+		printErrorAtLine(oldModule, ERR_EXT_OPTION_REMOVED,
+				 smiGetOptionLine(oldOption),
+				 oldModule->name, oldOptionNode->name,
+				 oldNode->name);
+	    } else {
+		printErrorAtLine(oldModule, ERR_OPTION_REMOVED,
+				 smiGetOptionLine(oldOption),
+				 oldOptionNode->name,
+				 oldNode->name);
+	    }
+	} else {
+	    code = 0;
+	    code |= checkDescription(oldModule, smiGetOptionLine(oldOption),
+				     newModule, smiGetOptionLine(newOption),
+				     newOptionNode->name, SMI_DECL_COMPL_GROUP,
+				     oldOption->description,
+				     newOption->description);
+	    if (code & CODE_SHOW_PREVIOUS) {
+		printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
+				 smiGetOptionLine(oldOption),
+				 oldOptionNode->name);
+	    }
+	}
+	smiInit(oldTag);
+    }
+
+    smiInit(newTag);
+    for (newOption = smiGetFirstOption(newNode);
+	 newOption; newOption = smiGetNextOption(newOption)) {
+	newOptionNode = smiGetOptionNode(newOption);
+	smiInit(oldTag);
+	for (oldOption = smiGetFirstOption(oldNode);
+	     oldOption; oldOption = smiGetNextOption(oldOption)) {
+	    oldOptionNode = smiGetOptionNode(oldOption);
+	    if (strcmp(oldOptionNode->name, newOptionNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! oldOption) {
+	    if (strcmp(smiGetNodeModule(newOptionNode)->name,
+		       newModule->name)) {
+		printErrorAtLine(newModule, ERR_EXT_OPTION_ADDED,
+				 smiGetOptionLine(newOption),
+				 newModule->name, newOptionNode->name,
+				 newNode->name);
+	    } else {
+		printErrorAtLine(newModule, ERR_OPTION_ADDED,
+				 smiGetOptionLine(newOption),
+				 newOptionNode->name,
+				 newNode->name);
+	    }
+	}
+	smiInit(newTag);
+    }
+}
+
+
+
+static void
+checkComplRefinements(SmiModule *oldModule, const char *oldTag,
+		  SmiModule *newModule, const char *newTag,
+		  SmiNode *oldNode, SmiNode *newNode)
+{
+    int code;
+    SmiRefinement *oldRefinement, *newRefinement;
+    SmiNode *oldRefinementNode, *newRefinementNode;
+
+    smiInit(oldTag);
+    for (oldRefinement = smiGetFirstRefinement(oldNode);
+	 oldRefinement; oldRefinement = smiGetNextRefinement(oldRefinement)) {
+	oldRefinementNode = smiGetRefinementNode(oldRefinement);
+	smiInit(newTag);
+	for (newRefinement = smiGetFirstRefinement(newNode);
+	     newRefinement; newRefinement = smiGetNextRefinement(newRefinement)) {
+	    newRefinementNode = smiGetRefinementNode(newRefinement);
+	    if (strcmp(oldRefinementNode->name, newRefinementNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! newRefinement) {
+	    if (strcmp(smiGetNodeModule(oldRefinementNode)->name,
+		       oldModule->name)) {
+		printErrorAtLine(oldModule, ERR_EXT_REFINEMENT_REMOVED,
+				 smiGetRefinementLine(oldRefinement),
+				 oldModule->name, oldRefinementNode->name,
+				 oldNode->name);
+	    } else {
+		printErrorAtLine(oldModule, ERR_REFINEMENT_REMOVED,
+				 smiGetRefinementLine(oldRefinement),
+				 oldRefinementNode->name,
+				 oldNode->name);
+	    }
+	} else {
+	    code = 0;
+	    code |= checkDescription(oldModule, smiGetRefinementLine(oldRefinement),
+				     newModule, smiGetRefinementLine(newRefinement),
+				     newRefinementNode->name,
+				     SMI_DECL_COMPL_OBJECT,
+				     oldRefinement->description,
+				     newRefinement->description);
+
+	    code |= checkAccess(oldModule,
+				smiGetRefinementLine(oldRefinement),
+				newModule,
+				smiGetRefinementLine(newRefinement),
+				newRefinementNode->name,
+				oldRefinement->access, newRefinement->access);
+
+	    checkTypeCompatibility(oldModule, oldRefinementNode,
+				   smiGetRefinementType(oldRefinement),
+				   newModule, smiGetRefinementLine(newRefinement),
+				   smiGetRefinementType(newRefinement));
+	    
+	    checkTypeCompatibility(oldModule, oldRefinementNode,
+				   smiGetRefinementWriteType(oldRefinement),
+				   newModule, smiGetRefinementLine(newRefinement),
+				   smiGetRefinementWriteType(newRefinement));
+	    
+	    if (code & CODE_SHOW_PREVIOUS) {
+		printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
+				 smiGetRefinementLine(oldRefinement),
+				 oldRefinementNode->name);
+	    }
+	}
+	smiInit(oldTag);
+    }
+
+    smiInit(newTag);
+    for (newRefinement = smiGetFirstRefinement(newNode);
+	 newRefinement; newRefinement = smiGetNextRefinement(newRefinement)) {
+	newRefinementNode = smiGetRefinementNode(newRefinement);
+	smiInit(oldTag);
+	for (oldRefinement = smiGetFirstRefinement(oldNode);
+	     oldRefinement; oldRefinement = smiGetNextRefinement(oldRefinement)) {
+	    oldRefinementNode = smiGetRefinementNode(oldRefinement);
+	    if (strcmp(oldRefinementNode->name, newRefinementNode->name) == 0) {
+		break;
+	    }
+	}
+	if (! oldRefinement) {
+	    if (strcmp(smiGetNodeModule(newRefinementNode)->name,
+		       newModule->name)) {
+		printErrorAtLine(newModule, ERR_EXT_REFINEMENT_ADDED,
+				 smiGetRefinementLine(newRefinement),
+				 newModule->name, newRefinementNode->name,
+				 newNode->name);
+	    } else {
+		printErrorAtLine(newModule, ERR_REFINEMENT_ADDED,
+				 smiGetRefinementLine(newRefinement),
+				 newRefinementNode->name,
+				 newNode->name);
+	    }
+	}
+	smiInit(newTag);
+    }
+}
+
+
+
+static void
+checkCompliance(SmiModule *oldModule, const char *oldTag,
+		SmiModule *newModule, const char *newTag,
+		SmiNode *oldNode, SmiNode *newNode)
+{
+    int code = 0;
+    
+    code = checkName(oldModule, smiGetNodeLine(oldNode),
+		     newModule, smiGetNodeLine(newNode),
+		     oldNode->name, newNode->name);
+    
+    code |= checkDecl(oldModule, smiGetNodeLine(oldNode),
+		      newModule, smiGetNodeLine(newNode),
+		      newNode->name, oldNode->decl, newNode->decl);
+    
+    code |= checkStatus(oldModule, smiGetNodeLine(oldNode),
+			newModule, smiGetNodeLine(newNode),
+			newNode->name, oldNode->status, newNode->status);
+
+    code |= checkDescription(oldModule, smiGetNodeLine(oldNode),
+			     newModule, smiGetNodeLine(newNode),
+			     newNode->name, oldNode->decl,
+			     oldNode->description, newNode->description);
+
+    code |= checkReference(oldModule, smiGetNodeLine(oldNode),
+			   newModule, smiGetNodeLine(newNode),
+			   newNode->name,
+			   oldNode->reference, newNode->reference);
+
+    if (code & CODE_SHOW_PREVIOUS) {
+	printErrorAtLine(oldModule, ERR_PREVIOUS_DEFINITION,
+			 smiGetNodeLine(oldNode), oldNode->name);
+    }
+    if (code & CODE_SHOW_PREVIOUS_IMPLICIT) {
+	printErrorAtLine(oldModule, ERR_PREVIOUS_IMPLICIT_DEFINITION,
+			 smiGetNodeLine(oldNode));
+    }
+
+    checkComplMandatory(oldModule, oldTag, newModule, newTag,
+			oldNode, newNode);
+
+    checkComplOptions(oldModule, oldTag, newModule, newTag, oldNode, newNode);
+
+    checkComplRefinements(oldModule, oldTag, newModule, newTag, oldNode, newNode);
+}
+
+
+
+static void
+diffCompliances(SmiModule *oldModule, const char *oldTag,
+		SmiModule *newModule, const char *newTag)
+{
+    SmiNode *oldNode, *newNode;
+    
+    /*
+     * First check whether the old node definitions still exist and
+     * whether the updates (if any) are consistent with the SMI rules.
+     */
+    
+    smiInit(oldTag);
+    for(oldNode = smiGetFirstNode(oldModule, SMI_NODEKIND_COMPLIANCE);
+	oldNode;
+        oldNode = smiGetNextNode(oldNode, SMI_NODEKIND_COMPLIANCE)) {
+	smiInit(newTag);
+	newNode = smiGetNodeByOID(oldNode->oidlen, oldNode->oid);
+	if (newNode
+	    && newNode->oidlen == oldNode->oidlen
+	    && smiGetNodeModule(newNode) == newModule) {
+	    checkCompliance(oldModule, oldTag, newModule, newTag,
+			    oldNode, newNode);
+	} else {
+	    printErrorAtLine(oldModule, ERR_NODE_REMOVED,
+			     smiGetNodeLine(oldNode),
+			     getStringNodekind(oldNode->nodekind),
+			     oldNode->name);
+	}
+	smiInit(oldTag);
+    }
+
+    /*
+     * Let's see if there are any new definitions.
+     */
+
+    smiInit(newTag);
+    for (newNode = smiGetFirstNode(newModule, SMI_NODEKIND_COMPLIANCE);
+	 newNode;
+	 newNode = smiGetNextNode(newNode, SMI_NODEKIND_COMPLIANCE)) {
+	
+	smiInit(oldTag);
+	oldNode = smiGetNodeByOID(newNode->oidlen, newNode->oid);
+	if (! oldNode
+	    || newNode->oidlen != oldNode->oidlen
+	    || smiGetNodeModule(oldNode) != oldModule) {
+	    printErrorAtLine(newModule, ERR_NODE_ADDED,
+			     smiGetNodeLine(newNode),
+			     getStringNodekind(newNode->nodekind),
+			     newNode->name);
+	}
+	smiInit(newTag);
+    }
+}
+
+
+
 static void
 usage()
 {
@@ -2330,7 +2827,11 @@ static void help() { usage(); exit(0); }
 static void version() { printf("smidiff " SMI_VERSION_STRING "\n"); exit(0); }
 static void config(char *filename) { smiReadConfig(filename, "smidiff"); }
 static void level(int lev) { errorLevel = lev; }
-static void ignore(char *ign) { setErrorSeverity(ign, 9999); }
+static void ignore(char *ign)
+{
+    smiSetSeverity(ign, 9999);		/* libsmi  error messages */
+    setErrorSeverity(ign, 9999);	/* smidiff error messages */
+}
 
 static void preload(char *module) {
     smiInit(oldTag);
@@ -2403,7 +2904,7 @@ main(int argc, char *argv[])
     diffObjects(oldModule, oldTag, newModule, newTag);
     diffNotifications(oldModule, oldTag, newModule, newTag);
     diffGroups(oldModule, oldTag, newModule, newTag);
-    /* diffCompliances(oldModule, oldTag, newModule, newTag) */
+    diffCompliances(oldModule, oldTag, newModule, newTag);
 
     smiInit(oldTag);
     smiExit();
@@ -2411,5 +2912,10 @@ main(int argc, char *argv[])
     smiInit(newTag);
     smiExit();
 
+    if (fflush(stdout) || ferror(stdout)) {
+	perror("smidump: write error");
+	exit(1);
+    }
+
     return 0;
 }
